﻿using Microsoft.EntityFrameworkCore;
using Microsoft.EntityFrameworkCore.Storage;
using System.Data;
using System.Data.Common;
using System.Linq.Expressions;
using HealthMonitor.Core.Common;

namespace HealthMonitor.Core.Dal.EfCoreImpl
{
    public class EfCoreDataAccessor : IGpsCardDataAccessor,IHealthMonitorDataAccessor
    {
        private DbContext? _context;

        public string DataSchema
        {
            get
            {
                if (IsClose()) return string.Empty;
                return (_context as IWithDataSchema)!.DataSchema;
            }
        }

        public EfCoreDataAccessor(DbContext context)
        {
            _context = context;
        }

        //public EfCoreDataAccessor(ICollection<TableMappingRule> rules)
        //{
        //    if (contextType == null)
        //    {
        //        throw new ArgumentNullException("You have not set context type");
        //    }
        //    context = (DbContext)Activator.CreateInstance(contextType, rules);
        //}

        /// <summary>
        /// 是否已关闭
        /// </summary>
        /// <returns></returns>
        public bool IsClose()
        {
            return _context == null;
        }

        /// <summary>
        /// 关闭连接
        /// </summary>
        public void Close()
        {
            if (_context != null)
            {
                _context.Dispose();
                _context = null;
            }
        }

        public void Dispose()
        {
            this.Close();
        }

        #region 读操作
        /// <summary>
        /// 获取第一个符合条件的实体，或返回null
        /// </summary>
        /// <param name="expression"></param>
        /// <returns></returns>
        public T? GetFirstOrDefault<T>(Expression<Func<T, bool>> expression) where T : class
        {
            if (IsClose()) return default;
            return _context?.Set<T>().AsNoTracking().FirstOrDefault(expression);
        }

        public async Task<T> GetFirstOrDefaultAsync<T>(Expression<Func<T, bool>> expression) where T : class
        {
            if (IsClose()) return default!;

            var entity = await _context!.Set<T>().FirstOrDefaultAsync(expression);
            //var entity = await _context.Set<T>().FindAsync(values);
            if (entity != null) _context.Entry(entity).State = EntityState.Detached;    //取消实体跟踪

            return entity!;
        }

        /// <summary>
        /// 根据主键值来查询某条记录，单主键的表可直接输入主键，多主键的表注意主键次序
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="values">主键值</param>
        /// <returns>返回主键值为传入值的实体</returns>
        public T GetByID<T>(params object[] values) where T : class
        {
            if (IsClose()) return default!;

            var exp = BuildPrimaryKeyQueryExp<T>(values);
            if (exp == null) return default!;

            var entity = _context!.Set<T>().FirstOrDefault(exp);
            //var entity = _context.Set<T>().Find(values);
            if (entity != null) _context.Entry(entity).State = EntityState.Detached;   //取消实体跟踪

            return entity!;
        }

        /// <summary>
        /// 根据主键值来查询某条记录，单主键的表可直接输入主键，多主键的表注意主键次序
        /// </summary>
        /// <typeparam name="T">实体类型</typeparam>
        /// <param name="values">主键值</param>
        /// <returns>返回主键值为传入值的实体</returns>
        public async Task<T> GetByIDAsync<T>(params object[] values) where T : class
        {
            if (IsClose()) return default!;

            var exp = BuildPrimaryKeyQueryExp<T>(values);
            if (exp == null) return default!;

            var entity = await _context!.Set<T>().FirstOrDefaultAsync(exp);
            //var entity = await _context.Set<T>().FindAsync(values);
            if (entity != null) _context.Entry(entity).State = EntityState.Detached;    //取消实体跟踪

            return entity!;
        }

        /// <summary>
        /// 获取某个表的所有数据
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        public IQueryable<T> GetAll<T>() where T : class
        {
            if (IsClose()) return default!;
            var result = _context!.Set<T>().AsNoTracking().Where(s => true);
            return result;
        }

        /// <summary>
        /// 根据表达式进行查询返回结果
        /// </summary>
        /// <typeparam name="T">查询的类</typeparam>
        /// <param name="expression">查询表达式</param>
        /// <returns></returns>
        public IQueryable<T> GetMany<T>(Expression<Func<T, bool>> expression) where T : class
        {
            if (IsClose()) return default!;
            var result = _context!.Set<T>().AsNoTracking().Where(expression);
            return result;
        }

        /// <summary>
        /// 根据某个字段的值进行排序
        /// </summary>
        /// <typeparam name="T">排序后获得的集合的类型</typeparam>
        /// <typeparam name="TKey">排序字段的类型</typeparam>
        /// <param name="orderExpression">字段表达式 如：basicInfo下根据caseID排序（s=>s.caseID）</param>
        /// <param name="isASC">是否升序</param>
        /// <returns></returns>
        public IEnumerable<T> Order<T, TKey>(Func<T, TKey> orderExpression, bool isASC = false) where T : class
        {
            if (IsClose()) return default!;

            if (isASC)
                return _context!.Set<T>().AsNoTracking().OrderBy(orderExpression);
            else
                return _context!.Set<T>().AsNoTracking().OrderByDescending(orderExpression);
        }

        /// <summary>
        /// 在数据库中进行分页查询
        /// </summary>
        /// <typeparam name="T">查询的类</typeparam>
        /// <param name="pageSize">每页条数</param>
        /// <param name="pageIdx">当前页</param>
        /// <param name="expression">查询表达式</param>
        /// <returns></returns>
        public List<T> GetPageList<T, Tkey>(int pageSize, int pageIdx, Expression<Func<T, bool>> expression, Func<T, Tkey> orderExpression) where T : class
        {
            if (IsClose()) return default!;

            var datas = this.GetMany<T>(expression);
            var result = datas.OrderBy(orderExpression).Skip(pageSize * (pageIdx - 1)).Take(pageSize);
            return result.ToList();
        }

        /// <summary>
        /// 在数据库中进行分页查询
        /// </summary>
        /// <typeparam name="T">查询的类</typeparam>
        /// <param name="pageSize">页大小</param>
        /// <param name="pageIdx">页下标</param>
        /// <param name="expression">查询表达式</param>
        /// <param name="orderExpression">排序表达式</param>
        /// <returns>分页查询结果</returns>
        public async Task<List<T>> GetPageListAsync<T, Tkey>(int pageSize, int pageIdx, Expression<Func<T, bool>> expression, Func<T, Tkey> orderExpression) where T : class
        {
            if (IsClose()) return default!;

            var datas = this.GetMany<T>(expression);
            var result = datas.OrderBy(orderExpression).Skip(pageSize * (pageIdx - 1)).Take(pageSize).AsQueryable();
            var ret = await result.ToListAsync();
            return ret;
        }

        /// <summary>
        /// 获取条目数
        /// </summary>
        /// <typeparam name="T">查询的类</typeparam>
        /// <param name="expression">查询表达式</param>
        /// <returns>条目数</returns>
        public int GetCount<T>(Expression<Func<T, bool>> expression) where T : class
        {
            if (IsClose()) return default;

            int ret = 0;
            var query = this.GetMany<T>(expression);
            ret = query.Count();
            return ret;
        }

        /// <summary>
        /// 获取条目数
        /// </summary>
        /// <typeparam name="T">查询的类</typeparam>
        /// <param name="expression">查询表达式</param>
        /// <returns>条目数</returns>
        public async Task<int> GetCountAsync<T>(Expression<Func<T, bool>> expression) where T : class
        {
            if (IsClose()) return default;

            var query = this.GetMany<T>(expression);
            int ret = await query.CountAsync();
            return ret;
        }
        #endregion

        #region 写操作，并自动保存
        /// <summary>
        /// 插入数据，保存并更新其自增键
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="model"></param>
        /// <returns></returns>
        public T AddWithIdentity<T>(T model) where T : class
        {
            if (IsClose()) return default!;
            T retModel;

            try
            {
                if (model != null)
                {
                    _context!.Set<T>().Add(model);
                    _context.SaveChanges();
                    retModel = model;
                }
                else
                    retModel = null!;
            }
            catch (Exception)
            {
                retModel = null!;
            }

            return retModel;
        }

        /// <summary>
        /// 插入数据，保存并更新其自增键
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="model"></param>
        /// <returns></returns>
        public async Task<T> AddWithIdentityAsync<T>(T model) where T : class
        {
            if (IsClose()) return default!;
            T retModel;

            try
            {
                if (model != null)
                {
                    _context!.Set<T>().Add(model);
                    await _context.SaveChangesAsync();
                    retModel = model;
                }
                else
                    retModel = null!;
            }
            catch (Exception)
            {
                retModel = null!;
            }

            return retModel;
        }
        #endregion

        #region 写操作，需要手工调用Save触发保存
        /// <summary>
        /// 向上下文增加记录，但不保存，需要手动调用Save
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="model"></param>
        /// <returns></returns>
        public void Add<T>(T model) where T : class
        {
            if (IsClose()) return;
            if (model == null) return;
            _context!.Set<T>().Add(model);
        }

        public void Add(object model)
        {
            if (IsClose()) return;
            if (model == null) return;
            _context!.Add(model);
        }

        /// <summary>
        /// 批量插入实体
        /// </summary>
        /// <param name="models"></param>
        public void AddRange(IEnumerable<object> models)
        {
            if (IsClose()) return;
            if (models == null || models.Count() == 0) return;
            _context!.AddRange(models);
        }

        /// <summary>
        /// 按主键标记实体删除(即使传入的实体不是被追踪的实体，同主键的追踪实体依然会标记删除删除)
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="model"></param>
        /// <returns></returns>
        public void Delete<T>(T model) where T : class
        {
            if (IsClose()) return;
            if (model == null) return;
            try
            {
                var entity = _context!.Remove(model);
            }
            catch (InvalidOperationException e)
            {
                // 若在追踪列表中已经存在另一个实体和该实体有相同主键(但不是同一个)
                // 则找到追踪列表里的实体把它标记为删除
                Type modelType = typeof(T);
                var key = _context!.Model.FindEntityType(modelType)!.FindPrimaryKey();
                if (key == null)
                {
                    throw e;
                }
                else
                {
                    // 找主键
                    var props = key.Properties;
                    object[] param = new object[props.Count];
                    int idx = 0;
                    foreach (var p in props)
                    {
                        var clrProp = modelType.GetProperty(p.Name);
                        var val = clrProp!.GetValue(model);
                        param[idx] = val!;
                        idx++;
                    }
                    // 用主键找实体，标记为删除
                    var cacheModel = _context.Set<T>().Find(param);
                    if (cacheModel != null)
                    {
                        _context.Remove(cacheModel);
                    }
                }
            }
        }

        public void Delete(object model)
        {
            if (IsClose()) return;
            if (model == null) return;
            _context!.Remove(model);
        }

        /// <summary>
        ///批量删除实体
        /// </summary>
        /// <param name="models"></param>
        public void DeleteRanage(IEnumerable<object> models)
        {
            if (IsClose()) return;
            if (models == null || models.Count() == 0) return;
            _context!.RemoveRange(models);
        }

        /// <summary>
        /// 更新操作
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="model"></param>
        /// <returns></returns>
        public void Update<T>(T model) where T : class
        {
            if (IsClose()) return;
            if (model == null) return;
            var entity = _context!.Entry<T>(model);
            entity.State = EntityState.Modified;
        }

        /// <summary>
        /// 根据主键更新指定字段，使用时需要注意EFCore的仓储机制可能会使数据库和缓存仓的数据不一致。
        /// </summary>
        /// <typeparam name="T">映射类</typeparam>
        /// <typeparam name="TProperty">字段类型</typeparam>
        /// <param name="model">更新的实体</param>
        /// <param name="property">要更新的属性, VisualStudio在这里有Bug, 不能智能显示类型属性, 但不影响使用</param>
        /// <returns></returns>
        public void Update<T>(T model, Expression<Func<T, object>> property) where T : class
        {
            if (IsClose()) return;
            if (model == null) return;
            var entity = _context!.Entry<T>(model);
            entity.Property(property).IsModified = true;
        }

        public void Update(object model)
        {
            if (IsClose()) return;
            if (model == null) return;
            _context!.Update(model);
        }

        /// <summary>
        /// 批量更新实体
        /// </summary>
        /// <param name="models"></param>
        public void UpdateRanage(IEnumerable<object> models)
        {
            if (IsClose()) return;
            if (models == null || models.Count() == 0) return;
            _context!.UpdateRange(models);
        }
        #endregion

        /// <summary>
        /// 执行存储过程，返回存储过程中返回的数据表
        /// </summary>
        /// <param name="procName">存储过程名</param>
        /// <param name="parameters">参数</param>
        /// <returns>返回的数据表</returns>
        public List<DataTable> CallProcedure(string procName, params DbParameter[] parameters)
        {
            if (IsClose()) return default!;

            List<DataTable> ret = new List<DataTable>();
            var connection = _context!.Database.GetDbConnection();
            using DbCommand cmd = connection.CreateCommand();
            IDbContextTransaction tran = _context!.Database.CurrentTransaction!;
            if (tran != null)
            {
                cmd.Transaction = tran.GetDbTransaction();
            }

            cmd.CommandType = System.Data.CommandType.StoredProcedure;
            cmd.CommandText = procName;
            cmd.Parameters.AddRange(parameters);
            _context.Database.OpenConnection();
            DbDataReader? reader = cmd.ExecuteReader();
            DataTable? table = ReaderToDataTable(reader);
            ret.Add(table);
            while (reader.NextResult())
            {
                table = ReaderToDataTable(reader);
                ret.Add(table);
            }
            reader.Close();
            //context.Database.CloseConnection();

            return ret;
        }

        /// <summary>
        /// 执行存储过程，返回存储过程中返回的数据表
        /// </summary>
        /// <param name="procName">存储过程名</param>
        /// <param name="parameters">参数</param>
        /// <returns>返回的数据表</returns>
        public async Task<List<DataTable>?> CallProcedureAsync(string procName, params DbParameter[] parameters)
        {
            if (IsClose()) return default;

            List<DataTable> ret = new List<DataTable>();
            var connection = _context?.Database.GetDbConnection();
            DbDataReader? reader = null;
            DataTable? table = null;
            using DbCommand? cmd = connection!.CreateCommand();
            IDbContextTransaction? tran = _context?.Database.CurrentTransaction;
            if (tran != null)
            {
                cmd.Transaction = tran.GetDbTransaction();
            }

            cmd.CommandType = System.Data.CommandType.StoredProcedure;
            cmd.CommandText = procName;
            cmd.Parameters.AddRange(parameters);
            await _context!.Database.OpenConnectionAsync();
            reader = await cmd.ExecuteReaderAsync();
            table = ReaderToDataTable(reader);
            ret.Add(table);
            while (reader.NextResult())
            {
                table = ReaderToDataTable(reader);
                ret.Add(table);
            }
            reader.Close();
            //context.Database.CloseConnection();

            return ret;
        }

        /// <summary>
        /// 提交对数据进行的处理，如无处理返回-1
        /// </summary>
        /// <returns></returns>
        public int Save()
        {
            if (IsClose()) return -1;
            return _context!.SaveChanges();

            //以后需要解决并发异常 DbUpdateConcurrencyException
        }

        /// <summary>
        /// 提交对数据进行的处理，如无处理返回-1
        /// </summary>
        /// <returns></returns>
        public async Task<int> SaveAsync()
        {
            if (IsClose()) return -1;
            return await _context!.SaveChangesAsync();

            //以后需要解决并发异常 DbUpdateConcurrencyException
        }

        ///// <summary>
        ///// 异步获取一个事务对象
        ///// </summary>
        ///// <returns></returns>
        //public async Task<IDbContextTransaction> BeginTransactionAsync()
        //{
        //    var ret = await _context.Database.BeginTransactionAsync();
        //    return ret;
        //}

        ///// <summary>
        ///// 获取一个事务对象
        ///// </summary>
        ///// <returns></returns>
        //public IDbContextTransaction BeginTransaction()
        //{
        //    return _context.Database.BeginTransaction();
        //}

        public DbContext GetDbContext()
        {
            return _context!;
        }

        public void UntrackEntities(IEnumerable<object> models)
        {
            if (IsClose()) return;
            if (models == null || models.Count() == 0) return;
            //取消当前对象的跟踪
            foreach (var e in models)
            {
                _context!.Entry(e).State = EntityState.Detached;
            }
        }

        public void UntrackEntities(object model)
        {
            if (IsClose()) return;
            if (model == null) return;
            _context!.Entry(model).State = EntityState.Detached;
        }

        private DataTable ReaderToDataTable(DbDataReader reader)
        {
            if (IsClose()) return default!;

            DataTable table = new DataTable();
            for (int i = 0; i < reader.FieldCount; i++)
            {
                DataColumn column = new()
                {
                    DataType = reader.GetFieldType(i),
                    ColumnName = reader.GetName(i)
                };
                table.Columns.Add(column);
            }

            while (reader.Read())
            {
                DataRow row = table.NewRow();
                for (int i = 0; i < reader.FieldCount; i++)
                {
                    row[i] = reader[i];
                }
                table.Rows.Add(row);
            }

            return table;
        }

        private Expression<Func<T, bool>> BuildPrimaryKeyQueryExp<T>(params object[] values) where T : class
        {
            if (values == null || values.Length == 0) return null!;

            var keys = _context!.Model.FindEntityType(typeof(T))!.FindPrimaryKey();
            var vals = values;
            if (keys!.Properties.Count > 1)
            {
                vals = (vals[0] + "").Trim().Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
                if (vals.Length != keys.Properties.Count) throw new ArgumentOutOfRangeException("主键参数数量不匹配");
            }

            var parameter = Expression.Parameter(typeof(T));
            var v = StringHelper.ConvertToType(vals[0] + "", keys.Properties[0].ClrType);
            var exp = Expression.Equal(Expression.Property(parameter, keys.Properties[0].PropertyInfo!), Expression.Constant(v, keys.Properties[0].ClrType));

            for (int i = 1; i < vals.Length; i++)
            {
                v = StringHelper.ConvertToType(vals[i] + "", keys.Properties[i].ClrType);
                var right = Expression.Equal(Expression.Property(parameter, keys.Properties[i].PropertyInfo!), Expression.Constant(v, keys.Properties[i].ClrType));
                exp = Expression.AndAlso(exp, right);
            }
            return Expression.Lambda<Func<T, bool>>(exp, parameter);
        }
    }
}
